# Offline 4e Compendium v3.5 and v4 High Level Design Doc #

Last updated: 2016-07-13

## 1. Introduction ##
### 1.1 Background ###

This program is for retrieving entries from official compendium of D&D 4th edition, store them locally, and let users browse and search them offline.

This program does not contain compendium data by itself, and saving them for personal use does not violate copyright.

The program has two independently executed parts: data downloader and data viewer.

### 1.2 Goals ###

Downloader:
1. A Java GUI program (single jar) that takes a login and download compendium entries.
2. Handles transform and indexing of downloaded data for viewer consumption.
3. Can export a complete viewer solution by itself.
4. Only save raw downloaded data to database; data transform and index are done on export, and can be enhanced by program update.
5. Not multi-lingual.  Target audience should all read English.

v3.5 Viewer:
1. Update, fix, enhance the unfinished v3 viewer, so that it can last till v4 is ready.
2. Last version to support IE and a bilingiual interface.
3. The rest of this doc is about v4 viewer.

v4 Viewer:
1. A HTML file that loads jsonp data from subdirectory.
2. Provide advanced search capability such as OR search, exclude term, phase search, regex search.
3. Responsive layout that fit mobile, pad, and desktop.  Printer and keyboard friendly.
4. Data clipboard, data tag, modifiable result list (can add or remove result).
5. Navigation between entries including lookup and associated class/feat/power.
6. No multi-lingual.  Data is *lots of* English, better keep UI consistent with data.

## 2. Design ##
### 2.1 Downloader Screens ###

The downloader use a tabbed page design with no menu or bar.
It has four states:
 * Idle
 * Loading data
 * Downloading (Running or Stopping)
 * Exporting (Running or Stopping)

Data is downloaded by running a WebView, and stored raw in SQLite database using SQLJet.
Both entry and listing are stored.  Category is fixed; no point make it dynamic, now that 4e has ceased development.

1. Data screen
  * [Label] Status
  * [Text input] Login name     [Text input] Login password
  * [List] Category list, downloaded count, total count
  * [Button] Left button - Download / Export / Stop      [Button] Right button - View / Exit

2. Option screen
  * [Number input] Download timeout
  * [Number input] Download interval
  * [Number input] Entry retry count (with 25k entries, error is bound to happen)
  * [Number input] Agent string
  * [Checkbox] Fix data              [Checkbox] Compress data
  * [Checkbox] Show debug tabs       [Button] Clear log
  * [Button] Clear downloaded data   [Button] Dump data
  * [Button] Check update (also show check result)

3. Log screen (Debug)
  * [Text area] Execution Log

4. WebView screen (Debug)
  * [WebView] The view actually used to download data.

5. License screen
  * [HTML area] Help, FAQ, and License

### 2.2 Download Workflow ###

Data is loaded by a JavaFX WebView and extracted with JavaScript.

1. Test login (see entry login step below)
2. Foreach category with no entry count
  3. Load Category xsl and xml.
  4. Apply xsl on xml to get entry info and save all.
  5. Update category count.
6. Foreach entry with no data
  7. Load the entry.
  8. If login screen -> try login -> stop if cannot login.
  9. Otherwise -> save entry data.

### 2.3 Viewer Screens ###

The viewer is an app-like offline webpage with multiple screens,
with a left sliding menu.

1. Fixed top / bottom header
 * Title
 * [Button] Menu with dynamic functions depending on page

2. Menu items
 * Search
 * My Lists
 * Options
 * Help
 * About

3. Search screen
 * [Menu] Add search result to current or new list
 * [Text] Search input
 * [Toggle] Search name / Search full
 * [Accordion] Category to search (Grouped by PC and DM)
 * [Accordion] Filter (If single category)
 * [Button] View all
 * [List] Search result (updated live)

4. Data (List) screen
 * [Select] All lists
 * [Menu] New, clone, clear, delete, or export list
 * [Button] Add entry to current list. (popup search form)
 * [List] Entries to view, with drag handle to reorder.
 * Double click entry to remove from list, or add/edit note and tag.

5. Option screen
 * [Checkbox] Toggle night mode.
 * [Checkbox] Toggle highlight.
 * [Checkbox] Toggle entry popup.
 * [Button] Clear Entry Note.    [Button] Clear Entry Tag.
 * [Number input] Page size. (max 999)
 * [Radio] Bar/Menu position: top or bottom.

6. Help screen
 * [Button] Start spotlight tour.
 * Search syntax

7. About screen
 * Version History
 * Link to Github
 * License

## 3. Data ##
### 3.1. Downloader raw data ###

These data are stored by downloader in SQLite.

When the downloader generate a viewer, it well compile, sort, and save the data into viewer's format.

1. Config (store database version; not much use)
 * key
 * value

2. Category list (fixed)
 * id (Compendium id)
 * name
 * count (total entry, not saved entry; -1 until whole entry listing is saved)
 * order (for export)
 * flelds (CSV Headers)

3. Entry list by category
 * id (Compendium id)
 * name
 * fields (CSV row of list field, except name)
 * data (Raw HTML)

### 3.1. Viewer data ###

Viewer data is hierarchical, using UTF-8 string as data id.

All data is loaded in and search in shared worker, to support data sharing between multiple tabs.

Listing is loaded to data thread lazily with Promise.
Data is loaded to data thread lazily with Promise, stored in a WeakMap.
When idle, the data thread will clean up old text, allowing memory to drop, until all text are dropped.
List data is also stored on data thread, so that all tabs share same list.

Name index is loaded to search worker, for instant search.
Full text index is loaded by search worker lazily, and is not released.

Data may or may not be stored in compressed form.  Number of files is fixed.

Each worker has a job queue; latest user initiated action has highest priority.
Requests caused by cancelled actions are removed from the queue.

1. Listing
 * id
 * name
 * field_headess
 * entries
   * id
   * name
   * fields (each field is string or string array)
   * last access date
   * data key

2. Entry (weakmap)
  * entry data key => data

3. Name list
 * id
 * name
 * category
 * full text (without flavour text, without item in item set, without associated powers)