package db4e.converter;

import db4e.data.Category;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RaceConverter extends Converter {

   private static final int ORIGIN = 0;
   private static final int ABILITY = 1;
   private static final int SIZE = 2;

   public RaceConverter ( Category category ) {
      super( category );
   }

   @Override protected void initialise() {
      category.fields = new String[]{ "Origin", "DescriptionAttribute", "Size", "SourceBook" };
      super.initialise();
   }

   private final Matcher regxAbility  = Pattern.compile( "<b>Ability scores</b>: ([^<]+)" ).matcher( "" );

   @Override protected void convertEntry () {
      meta( null, null, meta( 1 ), meta( 2 ) );
      super.convertEntry();
      // Origin column
      switch ( entry.getId() ) {
         case "race3":  // Eladrin
         case "race4":  // Elf
         case "race16": // Drow
         case "race20": // Gnome
         case "race44": // Wilden
         case "race56": // Moon Elf
         case "race57": // Sun Elf
         case "race58": // Wild Elf
         case "race59": // Wood Elf
         case "race60": // Hamadryad
         case "race61": // Pixie
         case "race62": // Satyr
         case "race64": // Llewyrr Elf
         case "race66": // Svirfneblin
            meta( ORIGIN, "Fey" );
            break;
         case "race14": // Changeling
            meta( ORIGIN, "Natural Shapechanger" );
            break;
         case "race23": // Kobold
         case "race67": // Bozak Draconian
         case "race68": // Kapak Draconian
            meta( ORIGIN, "Natural Reptile" );
            break;
         case "race26": // Shadar-kai
         case "race52": // Shade
            meta( ORIGIN, "Shadow" );
            break;
         case "race33": // Genasi
            meta( ORIGIN, "Elemental" );
            break;
         case "race35": // Deva
            meta( ORIGIN, "Immortal" );
            break;
         case "race47": // Revenant
         case "race53": // Vryloka
            meta( ORIGIN, "Living Undead" );
            break;
         case "race49": // Shardmind
            meta( ORIGIN, "Immortal Construct" );
            break;
         case "race65": // Hengeyokai
            meta( ORIGIN, "Fey Shapechanger" );
            break;
         default:
            meta( ORIGIN, "Natural" );
      }
      // Ability column
      if ( find( regxAbility ) ) {
         String ability = regxAbility.group(1).replace( "+2 ", "" );
         if ( ability.endsWith( "choice" ) )
            meta( ABILITY, "Any" );
         else
            meta( ABILITY, shortenAbility( ability ) );

      } else {
         switch ( entry.getId() ) {
            case "race54" : // Gold Dwarf
            case "race55" : // Shield Dwarf
               meta( ABILITY, "Con, Str or Wis" );
               break;
            case "race56" : // Moon Elf
            case "race57" : // Sun Elf
            case "race64" : // Llewyrr Elf
               meta( ABILITY, "Int, Cha or Dex" );
               break;
            case "race58" : // Wild Elf
            case "race59" : // Wood Elf
               meta( ABILITY, "Dex, Int or Wis" );
               break;
            case "race67" : // Bozak Draconian
               meta( ABILITY, "Cha, Con or Str" );
               break;
            case "race68" : // Kapak Draconian
               meta( ABILITY, "Dex, Cha" );
               break;
         }
         fix( "missing meta" );
      }
      // Size column
      if ( meta( SIZE ).isEmpty() ) {
         meta( SIZE, "Medium" );
         fix( "missing meta" );
      }
   }
}